@@ -8,9 +8,9 @@
 (use-modules (nyacc lang c99 pprint))
 (use-modules (nyacc lang c99 munge))
 (use-modules (nyacc lang c99 cxeval))
-(use-modules (nyacc lang c99 util1))
-(use-modules (nyacc lang util))
+(use-modules (nyacc lang c99 util))
 (use-modules (nyacc lang sx-util))
+(use-modules (nyacc lang util))
 (use-modules (nyacc lex))
 (use-modules (nyacc util))
 (use-modules (ice-9 pretty-print))
@@ -65,8 +65,8 @@
       ))))
 
 (define mode 'file)
-(define mode 'code)
 (define mode 'decl)
+(define mode 'code)
 (define debug #f)
 (define xdef? (lambda (name mode) (memq mode '(code decl))))
 
@@ -90,7 +90,7 @@
 		 #:inc-dirs inc-dirs 
 		 #:inc-help inc-help
 		 #:show-incs #f
-		 #:mode mode #:debug #f
+ 		 #:mode mode #:debug #f
 		 #:xdef? xdef?))))
 
 (define (parse-string-list . str-l)
@@ -109,30 +109,50 @@
 ;;(ppsx inc-dirs)
 ;;(ppsx inc-help)
 
+;; see c99-06.test
+(define (expand-typerefs-in-code code indx)
+  (let* ((tree (parse-string code))
+	 (udict (c99-trans-unit->udict tree))
+	 (decl (and=> ((sxpath `((decl ,indx))) tree) car))
+	 (xdecl (expand-typerefs decl udict)))
+    xdecl))
+
 (let* ((code (string-append
-	      "struct foo { int a; double b; };\n"
-	      "struct foo x; /* hmmm */ \n"
+	      ;;"struct event { int events; void *data; }\n"
+	      ;;"  __attribute__ ((__packed__));\n"
+	      
+	      ;;"typedef int case04 __attribute__ ((__deprecated__));\n"
+	      
+	      ;;"typedef int *bla_t[2]; bla_t foo(bla_t (*)(bla_t));\n"
+
+	      ;;"struct foo { int a; double b; } __attribute__((__packed__));\n"
+	      "struct foo { int a; double b; }"
+	      ;;" __attribute__ ((__packed__))"
+	      ";\n"
+	      "struct foo x;\n"
 	      ))
-       ;;(tree (parse-string code))
-       (tree (parse-file "zz.c"))
-       (udict (c99-trans-unit->udict/deep tree))
-       (udecl (udict-ref udict "gluSphere"))
+       (tree (parse-string code))
+       ;;(tree (parse-file "zz.c"))
+       ;;(udict (c99-trans-unit->udict/deep tree))
+       (decl1 (sx-ref tree 1))
+       (udict (unitize-decl decl1 '()))
+       ;;(udecl (udict-struct-ref udict "epoll_event"))
        ;;(udecl (stripdown-udecl udecl))
        ;;(udecl (expand-typerefs udecl udict))
        ;;(mdecl (udecl->mspec/comm udecl))
        ;;(udecl (unitize-decl decl))
+       ;;(xdecl (expand-typerefs-in-code code 2))
        )
-  ;;(pp (get-gcc-cpp-defs))
-  ;;(pp tree)
-  ;;(pp udict)
-  (pp udecl)
+  (pp tree)
+  (pp udict)
+  ;;(pp udecl)
   ;;(pp mdecl)
-  ;;(pp (parse-string "int ***x;"))
+  ;;(pp xdecl)
   ;;(ppsx udecl)
   ;;(pp99 xdecl)
   ;;(ppsx (eval-c99-cx tree))
-  ;;(ppsx (get-gcc-inc-dirs))
-  ;;(pp cpp-defs)
+  ;;(pp (get-gcc-cpp-defs))
+  ;;(pp (get-gcc-inc-dirs))
   #t)
 
 ;; --- last line ---
