@@ -109,17 +109,18 @@
    ((char=? ch #\.) (read-char) (read-char) "...") ; assumes correct syntax
    (else #f)))
 
-;; @deffn {Procedure} find-incl-in-dirl file dirl next => path
+;; @deffn {Procedure} find-incl-in-dirl file dirl [next] => path
 ;; Find path to include file expression, (i.e., @code{<foo.h>} or
 ;; @code{"foo.h"}.  If @code{"foo.h"} form look in current directory first.
-;; If @var{next} is true then remove current directory from search path.
+;; If @var{next} (default false) is true then remove current directory from 
+;; search path.
 ;; @*Refs:
 ;; @itemize
 ;; @item https://gcc.gnu.org/onlinedocs/cpp/Search-Path.html
 ;; @item https://gcc.gnu.org/onlinedocs/cpp/Wrapper-Headers.html
 ;; @end itemize
 ;; @end deffn
-(define (find-incl-in-dirl file dirl next)
+(define* (find-incl-in-dirl file dirl #:optional (next #f))
   (let* ((cid (and=> (port-filename (current-input-port)) dirname))
 	 (file-type (string-ref file 0)) ;; #\< or #\"
 	 (file-name (substring file 1 (1- (string-length file))))
@@ -357,6 +358,9 @@
 	 (iter stl chl nxt rest))
 	(($space . ,rest)
 	 (iter stl (cons #\space chl) nxt rest))
+	((($comm . ,val) . ,rest)
+	 ;; replace comment with extra trailing space
+	 (iter stl chl (string-append "/*" val "*/ ") rest))
 	((,asis . ,rest)
 	 (iter stl chl asis rest))
 	(,otherwise
@@ -400,7 +404,7 @@
 	 (else (iter2 (cons (car chl) res) (cdr chl))))))
      (else (iter (cons ch chl) (read-char))))))
 
-(define (scan-cpp-input defs used end-tok)
+(define* (scan-cpp-input defs used end-tok #:key (keep-comments #t))
   ;; Works like this: scan for tokens (comments, parens, strings, char's, etc).
   ;; Tokens are collected in a (reverse ordered) list (tkl) and merged together
   ;; to a string on return using @code{rtokl->string}.  Keep track of expanded
@@ -433,9 +437,15 @@
       (unread-char ch) (finish rr tkl))
      ((and end-tok (char=? #\) ch) (zero? lv))
       (unread-char ch) (finish rr tkl))
-     ((or (char-set-contains? c:ws ch)	; whitespace
-	  (read-c-comm ch #f))		; comment
+     ((char-set-contains? c:ws ch)	; whitespace
       (iter rr (cons '$space tkl) lv (skip-il-ws (read-char))))
+     ((read-c-comm ch #f) =>		; comment
+      (lambda (comm)
+	;; Normally comments in CPP def's are replaced by a space.  We allow
+	;; comments to get passed through, hoping this does not break code.
+	(if keep-comments
+	    (iter rr (acons '$comm (cdr comm) tkl) lv (skip-il-ws (read-char)))
+	    (iter rr (cons '$space tkl) lv (skip-il-ws (read-char))))))
      ((read-c-ident ch) =>
       (lambda (iden)
 	(cond
