@@ -6,13 +6,13 @@ struct __packed__ case02;
 
 typedef struct __packed__ { int *ip; double x; } case03;
 
+#if 0
 typedef int case04 __attribute__ ((__deprecated__));
 
 extern double case05(int x) 
    __attribute__ ((__nothrow__, __leaf__))
    __attribute__ ((__const__));
 
-
 void (__attribute__((noreturn)) ****case06) (void);
 char *__attribute__((aligned(8))) *case07;
 
@@ -32,4 +32,7 @@ struct case15 { int x[2] __attribute__ ((aligned (8))); };
 
 short case16[3] __attribute__ ((aligned (__BIGGEST_ALIGNMENT__)));
 
+struct event { int events; void *data; } __attribute__ ((__packed__));
+#endif
+
 /* --- last line --- */
